commit 47df3ddedd22c3f8e68aff831edb7921937674a2
Author: Jan Kara <jack@suse.cz>
Date:   Wed Sep 11 14:22:22 2013 -0700

    writeback: fix occasional slow sync(1)
    
    In case when system contains no dirty pages, wakeup_flusher_threads() will
    submit WB_SYNC_NONE writeback for 0 pages so wb_writeback() exits
    immediately without doing anything, even though there are dirty inodes in
    the system.  Thus sync(1) will write all the dirty inodes from a
    WB_SYNC_ALL writeback pass which is slow.
    
    Fix the problem by using get_nr_dirty_pages() in wakeup_flusher_threads()
    instead of calculating number of dirty pages manually.  That function also
    takes number of dirty inodes into account.
    
    Signed-off-by: Jan Kara <jack@suse.cz>
    Reported-by: Paul Taysom <taysom@chromium.org>
    Cc: Wu Fengguang <fengguang.wu@intel.com>
    Signed-off-by: Andrew Morton <akpm@linux-foundation.org>
    Signed-off-by: Linus Torvalds <torvalds@linux-foundation.org>

diff --git a/fs/fs-writeback.c b/fs/fs-writeback.c
index 68851ff..87d7781 100644
--- a/fs/fs-writeback.c
+++ b/fs/fs-writeback.c
@@ -1049,10 +1049,8 @@ void wakeup_flusher_threads(long nr_pages, enum wb_reason reason)
 {
 	struct backing_dev_info *bdi;
 
-	if (!nr_pages) {
-		nr_pages = global_page_state(NR_FILE_DIRTY) +
-				global_page_state(NR_UNSTABLE_NFS);
-	}
+	if (!nr_pages)
+		nr_pages = get_nr_dirty_pages();
 
 	rcu_read_lock();
 	list_for_each_entry_rcu(bdi, &bdi_list, bdi_list) {
